<?php

declare(strict_types=1);

namespace Drupal\ip_info\Services;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides a service for interacting with the IP.
 */
class IpInfo implements IpInfoInterface {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a new IpInfoAi service.
   *
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    Client $http_client,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('ip_info');
  }

  /**
   * {@inheritdoc}
   */
  public function getIpInformation(string $ip_address, string $url, string $api_name): ?array {
    try {
      $response = $this->httpClient->request('GET', $url);
      $status_code = $response->getStatusCode();

      if ($status_code === Response::HTTP_OK) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return $data ?: NULL;
      }
      else {
        $this->logger->error('@api_name API request failed with status code @code for ip address @ip', [
          '@api_name' => $api_name,
          '@code' => $status_code,
          '@ip' => $ip_address,
        ]);
        return NULL;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('@api_name API request failed with error: @error', [
        '@api_name' => $api_name,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

}
